@@ -1,4 +1,4 @@
-;    Initialization file for TinySCHEME 1.39
+;    Initialization file for TinySCHEME 1.41
 
 ; Per R5RS, up to four deep compositions should be defined
 (define (caar x) (car (car x)))
@@ -70,10 +70,21 @@
 (define (abs n) (if (>= n 0) n (- n)))
 (define (exact->inexact n) (* n 1.0))
 (define (<> n1 n2) (not (= n1 n2)))
+
+; min and max must return inexact if any arg is inexact; use (+ n 0.0)
 (define (max . lst)
-     (foldr (lambda (a b) (if (> a b) a b)) (car lst) (cdr lst)))
+  (foldr (lambda (a b)
+           (if (> a b)
+             (if (exact? b) a (+ a 0.0))
+             (if (exact? a) b (+ b 0.0))))
+         (car lst) (cdr lst)))
 (define (min . lst)
-     (foldr (lambda (a b) (if (< a b) a b)) (car lst) (cdr lst)))
+  (foldr (lambda (a b)
+           (if (< a b)
+             (if (exact? b) a (+ a 0.0))
+             (if (exact? a) b (+ b 0.0))))
+         (car lst) (cdr lst)))
+
 (define (succ x) (+ x 1))
 (define (pred x) (- x 1))
 (define gcd
@@ -129,17 +140,20 @@
 (define (string->anyatom str pred)
      (let* ((a (string->atom str)))
        (if (pred a) a
-     (error "string->xxx: not a xxx" a))))
+         (error "string->xxx: not a xxx" a))))
 
-(define (string->number str) (string->anyatom str number?))
+(define (string->number str . base)
+    (let ((n (string->atom str (if (null? base) 10 (car base)))))
+        (if (number? n) n #f)))
 
 (define (anyatom->string n pred)
   (if (pred n)
       (atom->string n)
       (error "xxx->string: not a xxx" n)))
 
+(define (number->string n . base)
+    (atom->string n (if (null? base) 10 (car base))))
 
-(define (number->string n) (anyatom->string n number?))
 
 (define (char-cmp? cmp a b)
      (cmp (char->integer a) (char->integer b)))
@@ -199,31 +213,31 @@
   (if (null? lists)
       (cons cars cdrs)
       (let ((car1 (caar lists))
-      (cdr1 (cdar lists)))
-  (unzip1-with-cdr-iterative
-   (cdr lists)
-   (append cars (list car1))
-   (append cdrs (list cdr1))))))
+            (cdr1 (cdar lists)))
+        (unzip1-with-cdr-iterative
+          (cdr lists)
+          (append cars (list car1))
+          (append cdrs (list cdr1))))))
 
 (define (map proc . lists)
   (if (null? lists)
       (apply proc)
       (if (null? (car lists))
-    '()
-    (let* ((unz (apply unzip1-with-cdr lists))
-     (cars (car unz))
-     (cdrs (cdr unz)))
-      (cons (apply proc cars) (apply map (cons proc cdrs)))))))
+        '()
+        (let* ((unz (apply unzip1-with-cdr lists))
+               (cars (car unz))
+               (cdrs (cdr unz)))
+          (cons (apply proc cars) (apply map (cons proc cdrs)))))))
 
 (define (for-each proc . lists)
   (if (null? lists)
       (apply proc)
       (if (null? (car lists))
-    #t
-    (let* ((unz (apply unzip1-with-cdr lists))
-     (cars (car unz))
-     (cdrs (cdr unz)))
-      (apply proc cars) (apply map (cons proc cdrs))))))
+        #t
+        (let* ((unz (apply unzip1-with-cdr lists))
+               (cars (car unz))
+               (cdrs (cdr unz)))
+          (apply proc cars) (apply map (cons proc cdrs))))))
 
 (define (list-tail x k)
     (if (zero? k)
@@ -327,23 +341,24 @@
 
 ;;;;;Helper for the dynamic-wind definition.  By Tom Breton (Tehom)
 (define (shared-tail x y)
-   (let (  (len-x (length x))
-	   (len-y (length y)))
+   (let ((len-x (length x))
+         (len-y (length y)))
       (define (shared-tail-helper x y)
-	 (if
-	    (eq? x y)
-	    x
-	    (shared-tail-helper (cdr x) (cdr y))))
+         (if
+            (eq? x y)
+            x
+            (shared-tail-helper (cdr x) (cdr y))))
+
       (cond
-	 ((> len-x len-y)
-	    (shared-tail-helper
-	       (list-tail x (- len-x len-y))
-	       y))
-	 ((< len-x len-y)
-	    (shared-tail-helper
-	       x
-	       (list-tail y (- len-y len-x))))
-	 (#t (shared-tail-helper x y)))))
+         ((> len-x len-y)
+            (shared-tail-helper
+               (list-tail x (- len-x len-y))
+               y))
+         ((< len-x len-y)
+            (shared-tail-helper
+               x
+               (list-tail y (- len-y len-x))))
+         (#t (shared-tail-helper x y)))))
 
 ;;;;;Dynamic-wind by Tom Breton (Tehom)
 
@@ -354,9 +369,9 @@
       ;;These functions are defined in the context of a private list of
       ;;pairs of before/after procs.
       (  (*active-windings* '())
-	 ;;We'll define some functions into the larger environment, so
-	 ;;we need to know it.
-	 (outer-env (current-environment)))
+         ;;We'll define some functions into the larger environment, so
+         ;;we need to know it.
+         (outer-env (current-environment)))
 
       ;;Poor-man's structure operations
       (define before-func car)
@@ -365,72 +380,72 @@
 
       ;;Manage active windings
       (define (activate-winding! new)
-	 ((before-func new))
-	 (set! *active-windings* (cons new *active-windings*)))
+         ((before-func new))
+         (set! *active-windings* (cons new *active-windings*)))
       (define (deactivate-top-winding!)
-	 (let ((old-top (car *active-windings*)))
-	    ;;Remove it from the list first so it's not active during its
-	    ;;own exit.
-	    (set! *active-windings* (cdr *active-windings*))
-	    ((after-func old-top))))
+         (let ((old-top (car *active-windings*)))
+            ;;Remove it from the list first so it's not active during its
+            ;;own exit.
+            (set! *active-windings* (cdr *active-windings*))
+            ((after-func old-top))))
 
       (define (set-active-windings! new-ws)
-	 (unless (eq? new-ws *active-windings*)
-	    (let ((shared (shared-tail new-ws *active-windings*)))
+         (unless (eq? new-ws *active-windings*)
+            (let ((shared (shared-tail new-ws *active-windings*)))
 
-	       ;;Define the looping functions.
-	       ;;Exit the old list.  Do deeper ones last.  Don't do
-	       ;;any shared ones.
-	       (define (pop-many)
-		  (unless (eq? *active-windings* shared)
-		     (deactivate-top-winding!)
-		     (pop-many)))
-	       ;;Enter the new list.  Do deeper ones first so that the
-	       ;;deeper windings will already be active.  Don't do any
-	       ;;shared ones.
-	       (define (push-many new-ws)
-		  (unless (eq? new-ws shared)
-		     (push-many (cdr new-ws))
-		     (activate-winding! (car new-ws))))
+               ;;Define the looping functions.
+               ;;Exit the old list.  Do deeper ones last.  Don't do
+               ;;any shared ones.
+               (define (pop-many)
+                  (unless (eq? *active-windings* shared)
+                     (deactivate-top-winding!)
+                     (pop-many)))
+               ;;Enter the new list.  Do deeper ones first so that the
+               ;;deeper windings will already be active.  Don't do any
+               ;;shared ones.
+               (define (push-many new-ws)
+                  (unless (eq? new-ws shared)
+                     (push-many (cdr new-ws))
+                     (activate-winding! (car new-ws))))
 
-	       ;;Do it.
-	       (pop-many)
-	       (push-many new-ws))))
+               ;;Do it.
+               (pop-many)
+               (push-many new-ws))))
 
       ;;The definitions themselves.
       (eval
-	 `(define call-with-current-continuation
-	     ;;It internally uses the built-in call/cc, so capture it.
-	     ,(let ((old-c/cc call-with-current-continuation))
-		 (lambda (func)
-		    ;;Use old call/cc to get the continuation.
-		    (old-c/cc
-		       (lambda (continuation)
-			  ;;Call func with not the continuation itself
-			  ;;but a procedure that adjusts the active
-			  ;;windings to what they were when we made
-			  ;;this, and only then calls the
-			  ;;continuation.
-			  (func
-			     (let ((current-ws *active-windings*))
-				(lambda (x)
-				   (set-active-windings! current-ws)
-				   (continuation x)))))))))
-	 outer-env)
+         `(define call-with-current-continuation
+             ;;It internally uses the built-in call/cc, so capture it.
+             ,(let ((old-c/cc call-with-current-continuation))
+                 (lambda (func)
+                    ;;Use old call/cc to get the continuation.
+                    (old-c/cc
+                       (lambda (continuation)
+                          ;;Call func with not the continuation itself
+                          ;;but a procedure that adjusts the active
+                          ;;windings to what they were when we made
+                          ;;this, and only then calls the
+                          ;;continuation.
+                          (func
+                             (let ((current-ws *active-windings*))
+                                (lambda (x)
+                                   (set-active-windings! current-ws)
+                                   (continuation x)))))))))
+         outer-env)
       ;;We can't just say "define (dynamic-wind before thunk after)"
       ;;because the lambda it's defined to lives in this environment,
       ;;not in the global environment.
       (eval
-	 `(define dynamic-wind
-	     ,(lambda (before thunk after)
-		 ;;Make a new winding
-		 (activate-winding! (make-winding before after))
-		 (let ((result (thunk)))
-		    ;;Get rid of the new winding.
-		    (deactivate-top-winding!)
-		    ;;The return value is that of thunk.
-		    result)))
-	 outer-env)))
+         `(define dynamic-wind
+             ,(lambda (before thunk after)
+                 ;;Make a new winding
+                 (activate-winding! (make-winding before after))
+                 (let ((result (thunk)))
+                    ;;Get rid of the new winding.
+                    (deactivate-top-winding!)
+                    ;;The return value is that of thunk.
+                    result)))
+         outer-env)))
 
 (define call/cc call-with-current-continuation)
 
@@ -577,8 +592,8 @@
   (let* ((env (if (null? envl) (current-environment) (eval (car envl))))
          (xval (eval x env)))
     (if (closure? xval)
-  (make-closure (get-closure-code xval) env)
-  xval)))
+      (make-closure (get-closure-code xval) env)
+      xval)))
 
 ; Redefine this if you install another package infrastructure
 ; Also redefine 'package'
@@ -685,16 +700,17 @@
   (foldr (lambda (x y) (or (cond-eval x) (cond-eval y))) #f cond-list))
 
 (define (cond-eval condition)
-  (cond ((symbol? condition)
-   (if (member condition *features*) #t #f))
-  ((eq? condition #t) #t)
-  ((eq? condition #f) #f)
-  (else (case (car condition)
-    ((and) (cond-eval-and (cdr condition)))
-    ((or) (cond-eval-or (cdr condition)))
-    ((not) (if (not (null? (cddr condition)))
-         (error "cond-expand : 'not' takes 1 argument")
-         (not (cond-eval (cadr condition)))))
-    (else (error "cond-expand : unknown operator" (car condition)))))))
+  (cond
+    ((symbol? condition)
+       (if (member condition *features*) #t #f))
+    ((eq? condition #t) #t)
+    ((eq? condition #f) #f)
+    (else (case (car condition)
+            ((and) (cond-eval-and (cdr condition)))
+            ((or) (cond-eval-or (cdr condition)))
+            ((not) (if (not (null? (cddr condition)))
+                     (error "cond-expand : 'not' takes 1 argument")
+                     (not (cond-eval (cadr condition)))))
+            (else (error "cond-expand : unknown operator" (car condition)))))))
 
 (gc-verbose #f)
